/*! \mainpage Task Description
 *
 * Implement the functions below.
 *
 * Unless stated otherwise, all the functions are expected to return nothing.
 *
 * @file
 */

#include <stdexcept>
#include <cstring>
#include <math.h>
#include "solution.hpp"

/**
 * Sets the values of all elements in an array of integers to a given value.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must throw an instance of std::runtime_error.
 *
 * @param begin a pointer to the first element in the array
 * @param end a pointer to the last element in the array
 * @param value an integer value to fill array elements with
 */
void my::fillIntegerArray(int *begin, int *end, int value) {}

/**
 * Use the memset function to set the values of all elements in an array of characters to a given value.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must throw an instance of std::runtime_error.
 *
 * @param begin a pointer to the first element in the array
 * @param end a pointer to the last element in the array
 * @param value a character value to fill array elements with
 */
void my::fillCharacterArray(char *begin, char *end, char value) {}

/**
 * Use the memcpy function to copy an array of integers from one location into another.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must throw an instance of std::runtime_error.
 *
 * It is safe to assume that the destination buffer is big enough to store all the elements in the source array.
 *
 * @param src_begin a pointer to the beginning of the source
 * @param src_end
 * @param dest
 */
void my::copyIntegerArray(int *src_begin, int *src_end, int *dest) {}


/**
 * Copy even elements from the source array of integers into a new dynamically allocated buffer.
 *
 * The pointers src_begin and src_end specify an inclusive range of elements. Hence, src_begin must never exceed src_end.
 *
 * In case an invalid argument is supplied, the function must throw an instance of std::runtime_error.
 *
 * @param src a pointer to the first integer in the source array
 * @param src_size the number of elements in the source array
 * @return a pair <a,b>, where a is a pointer to a newly created array, and b is the number of elements in the new array.
 */
std::pair<int*,std::size_t> my::copyEvenElements(int *src, std::size_t src_size) {
    return std::make_pair(nullptr,0);
}

/**
 * A given array is too small to accommodate the insertion of new elements into it.
 * Allocate a new memory block of size newsize, and copy all the elements from the old array over into the new one.
 * The remaining bytes in the new array, the ones not occupied by the bytes from the old array, must be set to value 0xFF.
 * Use the memcpy or memmove functions to copy the contents of the existing array to a new location.
 *
 * In case an invalid argument is supplied, the function must throw an instance of std::runtime_error.
 *
 * It is safe to assume that the newsize is never smaller than the size of the array to be copied.
 *
 * @param src a pointer to the first element in the array to be copied
 * @param size the number of elements in the array to be copied
 * @param newsize the number of elements in the new array
 * @return a pointer to the new array
 */
int* my::relocateIntegerArray(int *src, std::size_t size, std::size_t newsize) {
    return nullptr;
}

/**
 * Given an array of integers, calculate the sum of some of its elements.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 * The stride marks the offset. If stride is k, then calculate the sum over every k-th element, starting from the first one.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param begin a pointer to the first element in the array
 * @param end a pointer to the last element in the array
 * @param stride an integer. If stride = k >= 1, then we are interested in the sum over every k-th element in the array
 * @return the sum of the concerned elements, an integer
 */
int my::sumIntegerArray(int *begin, int *end, size_t stride) {
    return 0;
}

/**
 * Given an array of integers, calculate the mean of some of its elements.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 * The stride marks the offset. If stride is k, then calculate the mean over every k-th element, starting from the first one.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param begin a pointer to the first element in the array
 * @param end a pointer to the last element in the array
 * @param stride an integer. If stride = k >= 1, then we are interested in the mean over every k-th element in the array
 * @return the mean of the concerned elements, a single precision floating point value
 */
float my::integerArrayMean(int *begin, int *end, std::size_t stride) {
    return 0;
}

/**
 * Locate the an element in an array of integers.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param begin a pointer to the first element in the array
 * @param end a pointer to the last element in the array
 * @param target the value to search, an integer
 * @return the position (index) of the first occurrence of the concerned element (if found), -1 if the element was not found
 */
int my::findFirstElement(int *begin, int *end, int target) {}

/**
 * Locate the an element in an array of integers.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param begin a pointer to the first element in the array
 * @param end a pointer to the last element in the array
 * @param target the value to search, an integer
 * @return the position (index) of the last occurrence of the concerned element (if found), -1 if the element was not found
 */
int my::findLastElement(int *begin, int *end, int target) {
    return 0;
}

/**
 * Verify if an array of characters is safe to print as a string using functions like cout.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param begin a pointer to the first character in the array
 * @param end a pointer to the last character in the array
 * @return true if the array contains a byte with value zero (the string termination byte), false otherwise
 */
bool my::stringIsSafeToPrint(char *begin, char *end) {
    return true;
}


/**
 * Verify if an array of characters is a palindrome.
 *
 * The pointers begin and end specify an inclusive range of elements. Hence, begin must never exceed end.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param begin a pointer to the first character in the array
 * @param end a pointer to the last character in the array
 * @return true if the array contains a palindrome, false otherwise
 */
bool my::isPalindrome(char *begin, char *end) {
    return true;
}

/**
 * Verify if a given string is a palindrome.
 *
 * Hint: you can write this function as a wrapper around isPalindrome function above.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param str a pointer to the first byte in the string
 * @return true if the string is a palindrome, false otherwise
 */
bool my::isPalindrome2(char* str) {
    return true;
}

/**
 * Use memmove to delete character in a string at a given position.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param str a pointer to the first byte in the string
 * @param idx the position of the character to delete
 */
void my::deleteCharacter(char *str, unsigned int idx) {}

/**
 * Use memmove to insert a character into a string at a given position.
 *
 * It is safe to assume that the memory allocated for the string is sufficient to accommodate one more character.
 *
 * In case an invalid argument is supplied, the function must return an instance of std::runtime_error.
 *
 * @param str a pointer to the first byte in the string
 * @param character a character to insert
 * @param idx the position at which the character needs to be inserted
 */
void my::insertCharacter(char *str, char character, unsigned int idx) {}

/**
 * Use memmove to insert one string into another at a given position.
 *
 * It is safe to assume that the memory allocated for the string is sufficient to accommodate the two strings together.
 *
 * @param dest_str a pointer to the first byte in the string, into which the source string is copied
 * @param src_str a pointer to the first byte in the string that is being copied
 * @param idx the position in the destination string at which the source string is inserted into the destination string
 */
void my::insertCharacters(char *dest_str, char *src_str, unsigned int idx) {}
