#include <gtest/gtest.h>
#include "solution.hpp"

TEST(DynamicMemory, fillIntegerArray____invalidRange1) {
    try {
        int data[5]{1,2,3,4,5};
        my::fillIntegerArray(nullptr,data,5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillIntegerArray____invalidRange2) {
    try {
        int data[5]{1,2,3,4,5};
        my::fillIntegerArray(data,nullptr,5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillIntegerArray____invalidRange3) {
    try {
        int data[5]{1,2,3,4,5};
        my::fillIntegerArray(&data[3],&data[2],5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillIntegerArray____validRange1) {
    try {
        int x = 2;
        my::fillIntegerArray(&x,&x,5);
        ASSERT_EQ(x,5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillIntegerArray____validRange2) {
    try {
        int x[5]{1,2,3,4,5};
        my::fillIntegerArray(x,x,5);
        my::fillIntegerArray(&x[3],&x[3],11);
        ASSERT_EQ(x[0],5);
        ASSERT_EQ(x[1],2);
        ASSERT_EQ(x[2],3);
        ASSERT_EQ(x[3],11);
        ASSERT_EQ(x[4],5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillIntegerArray____validRange3) {
    try {
        int x[5]{1,2,3,4,5};
        my::fillIntegerArray(&x[1],&x[3],10);
        ASSERT_EQ(x[0],1);
        ASSERT_EQ(x[1],10);
        ASSERT_EQ(x[2],10);
        ASSERT_EQ(x[3],10);
        ASSERT_EQ(x[4],5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillIntegerArray____validRange4) {
    try {
        int x[5]{1,2,3,4,5};
        my::fillIntegerArray(x,x,10);
        my::fillIntegerArray(&x[1],&x[3],11);
        my::fillIntegerArray(&x[3],&x[4],12);
        ASSERT_EQ(x[0],10);
        ASSERT_EQ(x[1],11);
        ASSERT_EQ(x[2],11);
        ASSERT_EQ(x[3],12);
        ASSERT_EQ(x[4],12);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____invalidRange1) {
    try {
        char data[5]{1,2,3,4,5};
        my::fillCharacterArray(nullptr,data,5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____invalidRange2) {
    try {
        char data[5]{1,2,3,4,5};
        my::fillCharacterArray(data,nullptr,5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____invalidRange3) {
    try {
        char data[5]{1,2,3,4,5};
        my::fillCharacterArray(&data[3],&data[2],5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____validRange1) {
    try {
        char x = 2;
        my::fillCharacterArray(&x,&x,5);
        ASSERT_EQ(x,5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____validRange2) {
    try {
        char x[5]{1,2,3,4,5};
        my::fillCharacterArray(x,x,5);
        my::fillCharacterArray(&x[3],&x[3],11);
        ASSERT_EQ(x[0],5);
        ASSERT_EQ(x[1],2);
        ASSERT_EQ(x[2],3);
        ASSERT_EQ(x[3],11);
        ASSERT_EQ(x[4],5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____validRange3) {
    try {
        char x[5]{1,2,3,4,5};
        my::fillCharacterArray(&x[1],&x[3],10);
        ASSERT_EQ(x[0],1);
        ASSERT_EQ(x[1],10);
        ASSERT_EQ(x[2],10);
        ASSERT_EQ(x[3],10);
        ASSERT_EQ(x[4],5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, fillCharacterArray____validRange4) {
    try {
        char x[5]{1,2,3,4,5};
        my::fillCharacterArray(x,x,10);
        my::fillCharacterArray(&x[1],&x[3],11);
        my::fillCharacterArray(&x[3],&x[4],12);
        ASSERT_EQ(x[0],10);
        ASSERT_EQ(x[1],11);
        ASSERT_EQ(x[2],11);
        ASSERT_EQ(x[3],12);
        ASSERT_EQ(x[4],12);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____invalidRange1) {
    try {
        int src[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(nullptr,src,dest);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____invalidRange2) {
    try {
        int data[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(data,nullptr,dest);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____invalidRange3) {
    try {
        int data[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(data,data,nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____invalidRange4) {
    try {
        int data[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(&data[3],&data[2],dest);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____validRange1) {
    try {
        int data[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(&data[2],&data[2],&dest[1]);
        ASSERT_EQ(dest[0],0);
        ASSERT_EQ(dest[1],3);
        ASSERT_EQ(dest[2],0);
        ASSERT_EQ(dest[3],0);
        ASSERT_EQ(dest[4],0);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____validRange2) {
    try {
        int data[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(&data[1],&data[3],&dest[2]);
        ASSERT_EQ(dest[0],0);
        ASSERT_EQ(dest[1],0);
        ASSERT_EQ(dest[2],2);
        ASSERT_EQ(dest[3],3);
        ASSERT_EQ(dest[4],4);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyIntegerArray____validRange3) {
    try {
        int data[5]{1,2,3,4,5};
        int dest[5]{0,0,0,0,0};
        my::copyIntegerArray(data,&data[4],dest);
        ASSERT_EQ(dest[0],1);
        ASSERT_EQ(dest[1],2);
        ASSERT_EQ(dest[2],3);
        ASSERT_EQ(dest[3],4);
        ASSERT_EQ(dest[4],5);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyEvenElements____invalidAddress) {
    try {
        int data[5]{1,2,3,4,5};
        auto result = my::copyEvenElements(nullptr,5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyEvenElements____zeroRange1) {
    try {
        int data[5]{1,2,3,4,5};
        auto result = my::copyEvenElements(data,0);
        ASSERT_NE(result.first,nullptr);
        EXPECT_EQ(result.second,0);
        delete[] result.first;
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyEvenElements____zeroRange2) {
    try {
        int data[5]{1,3,5,7,9};
        auto result = my::copyEvenElements(data,5);
        ASSERT_NE(result.first,nullptr);
        EXPECT_EQ(result.second,0);
        delete[] result.first;
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, copyEvenElements____validRange) {
    try {
        int data[5]{1,2,3,4,5};
        auto result = my::copyEvenElements(data,5);
        ASSERT_NE(result.first,nullptr);
        EXPECT_EQ(result.second,2);
        EXPECT_EQ(result.first[0],2);
        EXPECT_EQ(result.first[1],4);
        delete[] result.first;
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, relocateIntegerArray____invalidAddress) {
    try {
        int data[5]{1,2,3,4,5};
        auto result = my::relocateIntegerArray(nullptr,5,10);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, relocateIntegerArray____invalidSize) {
    try {
        int data[5]{1,2,3,4,5};
        auto result = my::relocateIntegerArray(data,10,5);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, relocateIntegerArray____validSize) {
    try {
        int data[5]{1,2,3,4,5};
        auto dest = my::relocateIntegerArray(data,5,10);
        ASSERT_NE(dest, nullptr);
        for (int i=0; i<10; i++) {
            if (i<5) ASSERT_EQ(dest[i], data[i]);
            else ASSERT_EQ(dest[i], 0xFFFFFFFF);
        }
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____invalidRange1) {
    try {
        int x;
        auto result = my::sumIntegerArray(nullptr,&x);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____invalidRange2) {
    try {
        int x;
        auto result = my::sumIntegerArray(&x,nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____invalidStride) {
    try {
        int x=0;
        auto result = my::sumIntegerArray(&x,&x,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____singleElementRange1) {
    try {
        int x{2};
        ASSERT_EQ(my::sumIntegerArray(&x,&x),2);
    } catch (std::runtime_error& e) {
        std::cout << "RUNTIME " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____singleElementRange2) {
    try {
        int x{5};
        ASSERT_EQ(my::sumIntegerArray(&x,&x,2),5);
    } catch (std::runtime_error& e) {
        std::cout << "RUNTIME " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____multiElementRange1) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::sumIntegerArray(x,&x[4]),15);
    } catch (std::runtime_error& e) {
        std::cout << "RUNTIME " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____multiElementRange2) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::sumIntegerArray(&x[1],&x[3]),9);
    } catch (std::runtime_error& e) {
        std::cout << "RUNTIME " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____multiElementRange3) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::sumIntegerArray(x,&x[4],2),9);
    } catch (std::runtime_error& e) {
        std::cout << "RUNTIME " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, sumIntegerArray____multiElementRange4) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::sumIntegerArray(x,&x[4],3),5);
    } catch (std::runtime_error& e) {
        std::cout << "RUNTIME " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____invalidRange1) {
    try {
        int x;
        auto result = my::integerArrayMean(nullptr,&x);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____invalidRange2) {
    try {
        int x;
        auto result = my::integerArrayMean(&x,nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____invalidStride) {
    try {
        int x=0;
        auto result = my::integerArrayMean(&x,&x,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____singleElementRange1) {
    try {
        int x{2};
        ASSERT_NEAR(my::integerArrayMean(&x,&x),2, 0.001);
    } catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____singleElementRange2) {
    try {
        int x{5};
        ASSERT_NEAR(my::integerArrayMean(&x,&x,3),5, 0.001);
    } catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____multiElementRange1) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_NEAR(my::integerArrayMean(x,&x[4]),3,0.001);
    } catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____multiElementRange2) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_NEAR(my::integerArrayMean(&x[1],&x[3]),3,0.001);
    } catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____multiElementRange3) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_NEAR(my::integerArrayMean(x,&x[4],2),3,0.001);
    } catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, integerArrayMean____multiElementRange4) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_NEAR(my::integerArrayMean(x,&x[4],3),2.5,0.001);
    } catch (...) {
        std::cerr << "EXCEPTION " << std::endl;
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findFirstElement____invalidRange1) {
    try {
        int x=0;
        auto result = my::findFirstElement(nullptr,&x,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findFirstElement____invalidRange2) {
    try {
        int x = 0;
        auto result = my::findFirstElement(&x,nullptr,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findFirstElement____invalidRange3) {
    try {
        int x[5]{1,2,3,4,5};
        auto result = my::findFirstElement(&x[3],&x[2],0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findFirstElement____nonexistentElement) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::findFirstElement(x,&x[4],0),-1);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findFirstElement____existingElement) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::findFirstElement(x,&x[4],5),4);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findLastElement____invalidRange1) {
    try {
        int x=0;
        auto result = my::findLastElement(nullptr,&x,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findLastElement____invalidRange2) {
    try {
        int x = 0;
        auto result = my::findLastElement(&x,nullptr,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findLastElement____invalidRange3) {
    try {
        int x[5]{1,2,3,4,5};
        auto result = my::findLastElement(&x[3],&x[2],0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findLastElement____nonexistentElement) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::findLastElement(x,&x[4],0),-1);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, findLastElement____existingElement) {
    try {
        int x[5]{1,2,3,4,5};
        ASSERT_EQ(my::findLastElement(x,&x[4],5),4);
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____invalidRange1) {
    try {
        char c=0;
        auto result = my::stringIsSafeToPrint(nullptr,&c);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____invalidRange2) {
    try {
        char c=0;
        auto result = my::stringIsSafeToPrint(&c,nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____invalidRange3) {
    try {
        char c[2]{'a','a'};
        auto result = my::stringIsSafeToPrint(&c[1],&c[0]);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____validRange1) {
    try {
        char c[3]{0,'a','a'};
        ASSERT_TRUE(my::stringIsSafeToPrint(c,&c[2]));
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____validRange2) {
    try {
        char c[5]{'a','b', 0, 'c', 'd'};
        ASSERT_TRUE(my::stringIsSafeToPrint(c,&c[4]));
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____validRange3) {
    try {
        char c[5]{'a','b', 'c', 'd', 0};
        ASSERT_TRUE(my::stringIsSafeToPrint(c,&c[4]));
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, stringIsSafeToPrint____validRange4) {
    try {
        char c[5]{'a','b', 'c', 'd', 'e'};
        ASSERT_FALSE(my::stringIsSafeToPrint(c,&c[4]));
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}


TEST(DynamicMemory, isPalindrome____invalidRange1) {
    try {
        char c=0;
        auto result = my::isPalindrome(nullptr,&c);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, isPalindrome____invalidRange2) {
    try {
        char c=0;
        auto result = my::isPalindrome(&c,nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, isPalindrome____invalidRange3) {
    try {
        char c[2]{'a','a'};
        auto result = my::isPalindrome(&c[1],&c[0]);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, isPalindrome____validRange) {
    try {
        char str0[5]{'a', 'b', 'c', 'b', 'a'};
        char str1[6]{'a','b','c','c','b','a'};
        char str2[5]{'a','b','c','d','a'};
        char str3[5]{'a','b','c','b','d'};
        ASSERT_TRUE(my::isPalindrome(str0,&str0[4]));
        ASSERT_TRUE(my::isPalindrome(str1,&str1[5]));
        ASSERT_FALSE(my::isPalindrome(str2,&str2[4]));
        ASSERT_FALSE(my::isPalindrome(str3,&str3[4]));
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, isPalindrome2____invalidRange) {
    try {
        auto result = my::isPalindrome2(nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, isPalindrome2____validRange) {
    try {
        char str0[6]{"abcba"};
        char str1[7]{"abccba"};
        char str2[6]{"abcda"};
        char str3[6]{"abcbd"};
        ASSERT_TRUE(my::isPalindrome2(str0));
        ASSERT_TRUE(my::isPalindrome2(str1));
        ASSERT_FALSE(my::isPalindrome2(str2));
        ASSERT_FALSE(my::isPalindrome2(str3));
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, deleteCharacter____invalidRange1) {
    try {
        my::deleteCharacter(nullptr,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}


TEST(DynamicMemory, deleteCharacter____invalidRange2) {
    try {
        char str[]{"Hello, World!"};
        my::deleteCharacter(str,50);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, deleteCharacter____validRange) {
    try {
        char str[]{"Hello, World!"};
        my::deleteCharacter(str,0);
        my::deleteCharacter(str,2);
        my::deleteCharacter(str,3);
        my::deleteCharacter(str,4);
        my::deleteCharacter(str,8);
        ASSERT_STREQ(str,"elo orld");
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, insertCharacter____invalidRange1) {
    try {
        my::insertCharacter(nullptr,'z',0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}


TEST(DynamicMemory, insertCharacter____invalidRange2) {
    try {
        char str[]{"Hello, World!"};
        my::insertCharacter(str,'z',50);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, insertCharacter____validRange) {
    try {
        char str[100];
        memset(str,0,sizeof(str));
        const char* content = "Hello, World!";
        strncpy(str,content,strlen(content));
        my::insertCharacter(str,'z',0);
        my::insertCharacter(str,'z',2);
        my::insertCharacter(str,'z',14);
        ASSERT_STREQ(str,"zHzello, Worldz!");
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, insertCharacters____invalidRange1) {
    try {
        char z;
        my::insertCharacters(nullptr,&z,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}


TEST(DynamicMemory, insertCharacters____invalidRange2) {
    try {
        char z;
        my::insertCharacters(&z,nullptr,0);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, insertCharacters____invalidRange3) {
    try {
        char src[]{"Hello"};
        char dest[10];
        my::insertCharacters(dest,src,10);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, insertCharacters____validRange) {
    try {
        char dest[100];
        memset(dest,0,sizeof(dest));
        const char* dest_content = "Hello!";
        strncpy(dest,dest_content,strlen(dest_content));
        char src[]{", World"};
        my::insertCharacters(dest,src,5);
        ASSERT_STREQ(dest,"Hello, World!");
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}


